// -*- C++ -*-
/*****************************************************************************
 * KWATCH --- KDE Log File Viewer
 *
 * $Id: klogwidget.cpp,v 0.9 2000/12/04 21:20:47 mj Exp $
 *
 * QMultiLineEdit based widget for viewing log files
 *
 *****************************************************************************
 * Copyright (C) 2000
 *  _____ _____
 * |     |___  |   Martin Junius             Internet:  mailto:mj@m-j-s.net
 * | | | |   | |   Radiumstr. 18                        http://www.m-j-s.net/
 * |_|_|_|@home|   D-51069 Koeln, Germany
 *
 * Based on kwatch 0.3, with the following copyright notice:
 *
 * Kwatch is Copyright (C) 1997,1998 Ralph C. Weichert and can be
 * distributed freely under the terms of the GNU General Public License
 * (see file COPYING)
 *****************************************************************************
 * This file is part of KWATCH.
 *
 * KWATCH is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * KWATCH is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with KWATCH; see the file COPYING.  If not, write to the Free
 * Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *****************************************************************************/


#include <string.h>
#include <fcntl.h>
#include <sys/stat.h>

#include <kurl.h>
#include <kapp.h>

#include "klogwidget.h"
#include "prefs.h"



#define maxlines 500
#define bufsize 200
#define numlines 10



/*
 * KLogFile
 */

KLogFile::KLogFile(const QString &name, QObject *parent) :
    QObject( parent )
{
    file = new QFile(name);
    activate();
}


KLogFile::~KLogFile()
{
    delete file;
}



void KLogFile::activate()
{
    if(file->isOpen())
	return;
    file->open( IO_ReadOnly | IO_Raw);
    if(file->isOpen())
        // Go to EOF
        file->at(file->size());
    else {
        // Try again in a minute
        QTimer::singleShot( 60000, this, SLOT(activate()) );
    }
}



bool KLogFile::readMore()
{
    char buf[bufsize];
    QString text;
    int len;

    if ( !file->isOpen() ) return FALSE;
    
    do {
        len = file->readBlock( buf, bufsize-1 );
        if (len < 0) {
            text += "\nRead Error!\n";
            file->close();
            QTimer::singleShot( 60000, this, SLOT(activate()) );
            break;
        }
        buf[len] = 0;
        text += buf;
    } while (len == bufsize-1);
    if ( !text.isEmpty() ) {
        emit moreLines( this, text );
        return TRUE;
    }
    return FALSE;
}




/*
 * KLogWidget
 */

KLogWidget::KLogWidget(QWidget *parent, kwatchConf &cf)
    : QMultiLineEdit(parent), m_cf(cf),
      m_timer(0), m_files(0), m_current(0),
      m_r(0), m_c(0)
{
    // set QMultiLineEdit parameters
    setReadOnly( TRUE );

    // set configuration parameters
    updateConf();

    // start timer
    m_timer = new QTimer( this );
    m_timer->start(0);
    connect( m_timer, SIGNAL(timeout()), SLOT(timeout()) );
}



KLogWidget::~KLogWidget()
{
    if(m_timer)
	delete m_timer;
    if(m_files)
	delete m_files;

}



void KLogWidget::timeout()
{
    bool text_added = FALSE;
    QListIterator<KLogFile> it(*m_files);

    for(; it.current(); ++it)
        if(it.current()->readMore())
            text_added = TRUE;
    if(text_added)
        m_timer->changeInterval(0);
    else
        m_timer->changeInterval(1000);
}



void KLogWidget::moreText(KLogFile *f, QString s)
{
    if (s.isEmpty()) return;

    QString more = s;

    if(f && f != m_current )
    {
        m_current = f;
        more.prepend(formatName(f->file->name()));
    }

    insertAt(more, m_r, m_c);
    for (int l = numLines() - maxlines; l > 0; l--)
        removeLine(0);
    m_r = numLines() - 1;
    while (0 == (m_c = lineLength(m_r)) && m_r > 0) m_r--;
    if (more.right(1) == "\n") m_r++;
    setCursorPosition(numLines()-1, 0);
}



QString KLogWidget::formatName(const QString &name)
{
    QString std = "---------------[" + name + "]---------------\n";

    QString s = textLine(0);
    if(s.isEmpty() || s.isNull())
	return std;
    
    int l    = textWidth(0);
    int cols = width() / (l / s.length());
    int n    = cols - name.length() - 15 - 1 - 1 - 4;
    QString f;
    f.fill('-', n);
    
    return "---------------[" + name + "]" + f + "\n";
}



void KLogWidget::updateConf()
{
    fprintf(stderr, "KLogWidget::updateConf()\n");

    // list of files
    if(m_files)
	delete m_files;
    m_files = new QList<KLogFile>();
    m_files->setAutoDelete(TRUE);
    m_current = NULL;

    QStringList::Iterator it;
    for (it=m_cf.logfiles.begin(); it!=m_cf.logfiles.end(); it++)
    {
	fprintf(stderr, "KLogWidget::readProperties() - addFile(%s)\n",
		(const char *)(*it));
        addFile((*it));
    }

    // set font, fg, bg colors
    kwatchPrefs::setFontFgBg(this, m_cf.font, m_cf.fg, m_cf.bg);
}



void KLogWidget::reopen()
{
    QListIterator<KLogFile> it(*m_files);
    KLogFile *f;

    for(; (f = it.current()); ++it)
    {
        if(f->file->isOpen())
	    f->file->close();
        f->activate();
    }
}



void KLogWidget::addFile(const QString &name)
{
    KLogFile *f = new KLogFile(name);
    if(! f->file->isOpen())
	moreText(f, "ERROR: cannot open file\n");
    connect(f, 
	    SIGNAL(moreLines(KLogFile *, QString)),
	    SLOT(moreText(KLogFile *, QString))    );
    m_files->append(f);
}



#include "klogwidget.moc"
